/*
 * libgpephone
 *
 * Widget and utility library for GPE Phone Edition.
 * 
 * (c) kernel concepts 2006, 2007
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>

#include <libintl.h>

#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-bindings.h>
#include "DbusApplauncherBindings.h"
#include "gpeskwindow.h"

#define _(x) gettext(x)

#define DBUS_SKWINDOW_IF   "org.gpe.skwindow.softkeys"
#define DBUS_SKWINDOW_PATH "/org/gpe/skwindow/softkeys"

static DBusGProxy *proxy_msg = NULL;
static DBusGProxy *proxy_softkeys = NULL;
 
/** 
 * skwindow_dbus_present_applauncher:
 * @nr: 0 for homescreen, 1 for main menu.
 * 
 *
 * Call remote service object to present applauncher.
 */
void skwindow_dbus_present_applauncher (gint nr)
{
  org_gpe_applauncher_msg_present (proxy_msg, nr, NULL); 
}

/** 
 * skwindow_dbus_set_softkey_label:
 * @nr: Softkey number.
 * @text: Label text.
 *
 * Call remote service object to set softkey label.
 */
void 
skwindow_dbus_set_softkey_label (gint nr, const gchar *text)
{
  org_gpe_applauncher_softkeys_set_label (proxy_softkeys, nr, text, NULL);
}

/** 
 * skwindow_dbus_get_softkey_amount:
 *
 * Call remote service object to get number of softkeys.
 *
 * Returns: Number of softkeys, 0 if not present or on failure.
 */
gint
skwindow_dbus_get_softkey_amount (void)
{
  gint32 ret = 0;
    
  org_gpe_applauncher_softkeys_get_number (proxy_softkeys, &ret, NULL);
    
  return (gint)ret;
}

/** 
 * skwindow_dbus_get_softkey_key:
 * @nr: Key index
 *
 * Call remote service object to get key binding of a softkey.
 *
 * Returns: Text description of key binding, NULL on failure.
 */
gchar *
skwindow_dbus_get_softkey_key (gint nr)
{
  char *ret;
	
  org_gpe_applauncher_softkeys_get_key (proxy_softkeys, nr, &ret, NULL);
	
  return (gchar*)ret;
}

/** 
 * skwindow_dbus_get_softkey_menupos:
 * @nr: Key index
 * @menu_x: return value for X position
 * @menu_y: return value for Y position
 *
 * Call remote service object to get the menu position for of a softkey.
 *
 * Returns: Configured positions.
 */
void 
skwindow_dbus_get_softkey_menupos (gint nr, gint *menu_x, gint *menu_y)
{
  *menu_x = 0;
  *menu_x = 0;
  org_gpe_applauncher_softkeys_get_menu_position (proxy_softkeys, nr, menu_x, menu_y, NULL);
}

static DBusHandlerResult
msg_filter (DBusConnection *connection, DBusMessage *message, void *user_data)
{  
  GpeSKWindow *window = user_data;
   
  /* handle notifications */ 
  if (dbus_message_is_signal (message, DBUS_SKWINDOW_IF, "softkey_action")) 
    {
      DBusError error;
      int i;
        
      dbus_error_init (&error);
      if (dbus_message_get_args (message, &error, DBUS_TYPE_INT32, &i,
                                 DBUS_TYPE_INVALID)) 
        {
	  gpe_skwindow_softkey_activated (window, i);
          g_print ("%d was received\n", i);
        } 
      else 
        {
          g_print("Message received, but error getting message: %s\n", error.message);
          dbus_error_free (&error);
        }
        return DBUS_HANDLER_RESULT_HANDLED;
    }
	
  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
}

/** 
 * skwindow_dbus_init:
 *
 * Initialize DBus interface and open bus connection.
 *
 * Returns: TRUE on success, FALSE on failure.
 */
gboolean
skwindow_dbus_init (GpeSKWindow *window)
{
  GError *error = NULL;
  DBusGConnection *connection;
  DBusConnection *bus;
  GObject *obj;
  guint32 request_name_ret;
 
  connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
  bus = dbus_bus_get (DBUS_BUS_SESSION, NULL);
  if (connection == NULL)
    {
      g_printerr ("Failed to get session bus connection %s", error->message);
      return FALSE;
    }
  
  /* we do not request a namespace to support multiple listeners to these signals */
  dbus_bus_add_match (bus, "type='signal',interface='" DBUS_SKWINDOW_IF "'", NULL);
  dbus_connection_add_filter (bus, msg_filter, window, NULL);
  
  /* msg object proxy  */
  proxy_msg = dbus_g_proxy_new_for_name (connection,
                                              DBUS_APPLAUNCHER_SERVICE_NAME,
                                              DBUS_APPLAUNCHER_MSG_PATH,
                                              DBUS_APPLAUNCHER_MSG_INTERFACE);

  /* softkeys object proxy  */
  proxy_softkeys = dbus_g_proxy_new_for_name (connection,
                                              DBUS_APPLAUNCHER_SERVICE_NAME,
                                              DBUS_APPLAUNCHER_SOFTKEYS_PATH,
                                              DBUS_APPLAUNCHER_SOFTKEYS_INTERFACE);

  
  if (proxy_softkeys == NULL)
    {
      g_printerr ("Failed to create bus proxy.");
      return FALSE;
    }
    
  return TRUE;
}
