/*
 * libgpephone
 *
 * Widget and utility library for GPE Phone Edition.
 * 
 * (c) kernel concepts 2006, 2007
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gpe-sktabs.h"

/* 'private'/'protected' functions */
static void            gpe_sktabs_class_init     (GpeSKTabsClass *klass);
static void            gpe_sktabs_init           (GpeSKTabs *obj);
static void            gpe_sktabs_finalize       (GObject *obj);

/* list my signals */
enum 
{
  LAST_SIGNAL
};

typedef struct _TabSKeydef  TabSKeydef;

struct _TabSKeydef
{
  gint      count;
  GtkWidget **menu_set;
  gchar     **label_set;    
};

typedef struct _GpeSKTabsPrivate GpeSKTabsPrivate;

/* private members */
struct _GpeSKTabsPrivate
{
  gint num_tabs;
  TabSKeydef *skeydefs;
};
#define GPE_SKTABS_GET_PRIVATE(o)      (G_TYPE_INSTANCE_GET_PRIVATE((o), \
                                        GPE_TYPE_SKTABS, \
                                        GpeSKTabsPrivate))
/* globals */
static GtkNotebookClass *parent_class = NULL;

GType
gpe_sktabs_get_type (void)
{
  static GType my_type = 0;
  if (!my_type) {
    static const GTypeInfo my_info = 
    {
            sizeof(GpeSKTabsClass),
            NULL,		/* base init */
            NULL,		/* base finalize */
            (GClassInitFunc) gpe_sktabs_class_init,
            NULL,		/* class finalize */
            NULL,		/* class data */
            sizeof(GpeSKTabs),
            1,		/* n_preallocs */
            (GInstanceInitFunc) gpe_sktabs_init,
    };
    my_type = g_type_register_static (GTK_TYPE_NOTEBOOK,
                                      "GpeSKTabs",
  	                                  &my_info, 0);
  }
  return my_type;
}

static void
gpe_sktabs_class_init (GpeSKTabsClass *klass)
{
  GObjectClass *gobject_class;
  gobject_class = (GObjectClass*) klass;

  parent_class            = g_type_class_peek_parent (klass);
  gobject_class->finalize = gpe_sktabs_finalize;

  g_type_class_add_private (gobject_class, sizeof(GpeSKTabsPrivate));

  klass->append_page_with_menus = gpe_sktabs_append_page_with_menus;
  klass->update_page_menus = gpe_sktabs_update_page_menus;
	
	/* signal definitions */
}

static void
gpe_sktabs_init (GpeSKTabs *obj)
{
  GpeSKTabsPrivate *priv = GPE_SKTABS_GET_PRIVATE(obj);

  priv->num_tabs = 0;
  priv->skeydefs = NULL;
    
  /* turn off tabs of parent */
}

static void
gpe_sktabs_finalize (GObject *obj)
{
  GpeSKTabsPrivate *priv = GPE_SKTABS_GET_PRIVATE(obj);
  gint i, j;
    
   for (i = 0; i < priv->num_tabs; i++)
      for (j = 0; j < priv->skeydefs[i].count; j++)
          if (priv->skeydefs[i].label_set[j])
              g_free (priv->skeydefs[i].label_set[j]);
   if (priv->num_tabs)
     {
         priv->num_tabs = 0;
         g_free (priv->skeydefs);
     }
}

static void
switch_page (GpeSKTabs *tabs, GtkNotebookPage *page, guint page_num, gpointer user_data)
{
  GpeSKWindow *window = user_data;
  GpeSKTabsPrivate *priv = GPE_SKTABS_GET_PRIVATE(tabs);
  gint i;
    
  g_return_if_fail (page_num <= priv->num_tabs);

  for (i = 0; i < priv->skeydefs[page_num].count; i++)
    {
      gpe_skwindow_set_softkey_label (window, i, priv->skeydefs[page_num].label_set[i]);
      gpe_skwindow_set_softkey_menu (window, i, priv->skeydefs[page_num].menu_set[i]);
    }
}

/** 
 * gpe_sktabs_new: 
 * @skwindow: A #GpeSKWindow to be used.
 *
 * Create a new #GpeSKTabs instance. 
 * 
 * Returns: A new #GpeSKTabs instance.
 */
GtkWidget*
gpe_sktabs_new (GpeSKWindow *skwindow)
{
  GpeSKTabs *sktabs;
    
  g_return_val_if_fail (GPE_IS_SKWINDOW (skwindow), NULL);
  sktabs = GPE_SKTABS (g_object_new(GPE_TYPE_SKTABS, NULL));  
  sktabs->skwindow = skwindow;
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (sktabs), FALSE); 
  g_signal_connect (G_OBJECT (sktabs), "switch-page", G_CALLBACK (switch_page), skwindow);
    
  return GTK_WIDGET (sktabs);
}

/** 
 * gpe_sktabs_append_page_with_menus:
 * @self: A #GpeSKTabs instance
 * @tablabel: A widget used as a label for tab decoration.
 * @content: Tab contents - usually a container widget.
 * @Varargs: Label text / menu widget pairs.
 *
 * This function appends a new page view to a #GpeSKTabs widget. 
 * The @tablabel argument may be %NULL if the tabs are not visible. 
 * The variable parameters are a null-terminated list of label text (gchar*)
 * and menu widget (GtkWidget*) items. You supply them in the native order 
 * of the available softkeys.
 * 
 */
void
gpe_sktabs_append_page_with_menus (GpeSKTabs *self, GtkWidget *tablabel, GtkWidget *content, ...)
{
  va_list var_args;
  gchar *label;
  GtkWidget *menu;
  gint pos, count;
  GpeSKTabsPrivate *priv = GPE_SKTABS_GET_PRIVATE(self);


  g_return_if_fail (GPE_IS_SKTABS (self));
  g_return_if_fail (content != NULL);

  count = 0;
  pos = priv->num_tabs;
  priv->num_tabs++;
  priv->skeydefs = g_realloc (priv->skeydefs, priv->num_tabs * sizeof (TabSKeydef));
  
  va_start (var_args, content);
  
  label = va_arg (var_args, gchar*);
  menu = va_arg (var_args, GtkWidget*);
    
  while (label && menu && GTK_IS_WIDGET (menu)) 
    {
        count ++;
        priv->skeydefs[pos].label_set = 
            g_realloc (priv->skeydefs[pos].label_set, count * sizeof (gchar*));
        priv->skeydefs[pos].label_set[count-1] = g_strdup (label);
        priv->skeydefs[pos].menu_set = 
            g_realloc (priv->skeydefs[pos].menu_set, count * sizeof (GtkWidget*));
        priv->skeydefs[pos].menu_set[count-1] = menu;
        g_object_ref (menu);
        priv->skeydefs[pos].count = count;
        
        label = va_arg (var_args, gchar*);
        menu = va_arg (var_args, GtkWidget*);
    }
  va_end (var_args);
  gtk_notebook_append_page (GTK_NOTEBOOK (self), content, tablabel);
}


/** 
 * gpe_sktabs_update_page_menus:
 * @self: A #GpeSKTabs instance
 * @page_num: A tab page index.
 * @Varargs: Label text / menu widget pairs.
 *
 * Modifies the softkey menues and labels assigned to a tab page 
 * of a #GpeSKTabs widget.
 * The variable parameters are a null-terminated list of label text (gchar*)
 * and menu widget (GtkWidget*) items. You supply them in the native order 
 * of the available softkeys.
 * 
 */
void
gpe_sktabs_update_page_menus (GpeSKTabs *self, gint page_num, ...)
{
  va_list var_args;
  gchar *label;
  GtkWidget *menu;
  gint count, i;
  GpeSKTabsPrivate *priv = GPE_SKTABS_GET_PRIVATE(self);


  g_return_if_fail (GPE_IS_SKTABS (self));
  g_return_if_fail (page_num >= 0 && page_num < priv->num_tabs);

  count = 0;
  
  for (i = 0; i < priv->skeydefs[page_num].count; i++)
      if (priv->skeydefs[page_num].label_set[i])
          g_free (priv->skeydefs[page_num].label_set[i]);
  g_free (priv->skeydefs[page_num].label_set);
  g_free (priv->skeydefs[page_num].menu_set);
  priv->skeydefs[page_num].label_set = NULL;
  priv->skeydefs[page_num].menu_set = NULL;
	
  va_start (var_args, page_num);
  
  label = va_arg (var_args, gchar*);
  menu = va_arg (var_args, GtkWidget*);
    
  while (label && menu && GTK_IS_WIDGET (menu)) 
    {
        count ++;
        priv->skeydefs[page_num].label_set = 
            g_realloc (priv->skeydefs[page_num].label_set, count * sizeof (gchar*));
        priv->skeydefs[page_num].label_set[count-1] = g_strdup (label);
        priv->skeydefs[page_num].menu_set = 
            g_realloc (priv->skeydefs[page_num].menu_set, count * sizeof (GtkWidget*));
        priv->skeydefs[page_num].menu_set[count-1] = menu;
        g_object_ref (menu);
        priv->skeydefs[page_num].count = count;
        
        label = va_arg (var_args, gchar*);
        menu = va_arg (var_args, GtkWidget*);
    }
  va_end (var_args);
  if (gtk_notebook_get_current_page (GTK_NOTEBOOK (self)) == page_num)
      switch_page (self, NULL, page_num, NULL);
}
