/*
 * libgpephone
 *
 * Widget and utility library for GPE Phone Edition.
 * 
 * (c) kernel concepts 2006, 2007
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GPE_TYPE_SKWINDOW, GpeSKWindowPrivate))

#include <libintl.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include "gpeskwindow.h"
#include "dbus-if.h"
#include <glib.h>

#define _(x) gettext(x)

typedef struct 
{
  guint key;
  gint menu_x;
  gint menu_y;
  GtkWidget *menu;
  gchar     *label;
} t_softkey;

typedef struct 
{
    gint      widget_disposed : 1;
    gint      have_softkeys   : 1;
    t_softkey *softkeys;
} GpeSKWindowPrivate;

enum {
   PROP_NONE,
   PROP_NUM_SOFTKEYS 
};

static gboolean show_flag = TRUE;
static void gpe_skwindow_class_init (GpeSKWindowClass *klass);
static void gpe_skwindow_init       (GpeSKWindow      *window);


static void gpe_skwindow_set_property  (GObject          *object,
                                        guint             prop_id,
                                        const GValue     *value,
                                        GParamSpec       *pspec);
static void gpe_skwindow_get_property  (GObject          *object,
                                        guint             prop_id,
                                        GValue           *value,
                                        GParamSpec       *pspec);

static void gpe_skwindow_map           (GtkWidget *window);
static void gpe_skwindow_close         (GpeSKWindow  *window);
static void gpe_skwindow_dispose       (GObject *obj);
static void gpe_skwindow_finalize      (GObject *obj);

enum {
  ACTIVATE_KEY,
  MENU_DOWN,
  LAST_SIGNAL
};

static gpointer parent_class;
static guint skwindow_signals[LAST_SIGNAL];

static gboolean menu_fullscreen = FALSE;

GType
gpe_skwindow_get_type (void)
{
  static GType skwindow_type = 0;

  if (!skwindow_type)
  {
    static const GTypeInfo skwindow_info =
    {
      sizeof (GpeSKWindowClass),
      NULL,		/* base_init */
      NULL,		/* base_finalize */
      (GClassInitFunc) gpe_skwindow_class_init,
      NULL,		/* class_finalize */
      NULL,		/* class_data */
      sizeof (GpeSKWindow),
      0,		/* n_preallocs */
      (GInstanceInitFunc) gpe_skwindow_init,
    };

    skwindow_type = g_type_register_static (GTK_TYPE_WINDOW, "GpeSKWindow", 
											&skwindow_info, 0);
  }

  return skwindow_type;
}

static void
gpe_skwindow_class_init (GpeSKWindowClass *class)
{
  GObjectClass *gobject_class;
  GtkWidgetClass *widget_class;
/*  GtkBindingSet *binding_set;*/
  
  gobject_class = G_OBJECT_CLASS (class);
  widget_class = GTK_WIDGET_CLASS (class);
  
  parent_class = g_type_class_peek_parent (class);

  gobject_class->set_property = gpe_skwindow_set_property;
  gobject_class->get_property = gpe_skwindow_get_property;
  gobject_class->dispose = gpe_skwindow_dispose;
  gobject_class->finalize = gpe_skwindow_finalize;
  
  widget_class->map = gpe_skwindow_map;

  class->close = gpe_skwindow_close;
  class->get_softkey_label = gpe_skwindow_get_softkey_label;
  class->set_softkey_label = gpe_skwindow_set_softkey_label;
  class->get_softkey_menu  = gpe_skwindow_get_softkey_menu;
  class->set_softkey_menu  = gpe_skwindow_set_softkey_menu;
  class->softkey_popup_menu  = gpe_skwindow_softkey_popup_menu;

  g_type_class_add_private (gobject_class, sizeof (GpeSKWindowPrivate));

  g_object_class_install_property (gobject_class,
                                   PROP_NUM_SOFTKEYS,
                                   g_param_spec_int ("num_softkeys",
							       _("Softkeys"),
							       _("The amount of softkeys available."),
                                   0,
                                   255,
                                   0,
                                   G_PARAM_READABLE));
  
  skwindow_signals[ACTIVATE_KEY] =
    g_signal_new ("softkey_activated",
		  G_OBJECT_CLASS_TYPE (class),
		  G_SIGNAL_RUN_LAST,
		  G_STRUCT_OFFSET (GpeSKWindowClass, softkey_activated),
		  NULL, NULL,
		  gtk_marshal_VOID__INT,
		  G_TYPE_NONE, 1,
		  G_TYPE_INT);
		  
  skwindow_signals[MENU_DOWN] =
    g_signal_new ("menu_down",
		  G_OBJECT_CLASS_TYPE (class),
		  G_SIGNAL_RUN_LAST,
		  G_STRUCT_OFFSET (GpeSKWindowClass, menu_down),
		  NULL, NULL,
		  gtk_marshal_VOID__INT,
		  G_TYPE_NONE, 1,
		  G_TYPE_INT);
  
/*  binding_set = gtk_binding_set_by_class (class);
  
    gtk_binding_entry_add_signal (binding_set, GDK_Escape, 0,
                                "close", 0);
*/
}

/* when the popup menu disappeared due to key press or mouse click, 
 * a signal would be sent to the application who owns the popup menu.
 * */
static void 
menu_pop_down (GtkWidget *widget, gpointer user_data)
{
#ifdef DEBUG
	g_message ("sending signal menu_down");
#endif
	gint key_number = GPOINTER_TO_INT (user_data);
	GpeSKWindow *window = g_object_get_data (G_OBJECT (widget), "window");
	g_signal_emit (window, skwindow_signals[MENU_DOWN], 0, key_number);
}
                                        
/* Softkey menu positioning function. This one determines the correct position
 * of the softkey menu. */
static void 
softkey_menu_position (GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
                       gpointer user_data)
{
  gint nr = (gint) user_data;
  GtkWidget *menu_item = user_data;  
  GpeSKWindow *window = g_object_get_data (G_OBJECT (menu), "window");
  GpeSKWindowPrivate *priv;
  GtkRequisition req;

  priv = GET_PRIVATE (window);
    
  gtk_widget_size_request (GTK_WIDGET (menu), &req);

  /* we assume only the last button is aligned right */
  if (nr != (window->num_softkeys - 1))
     *x = priv->softkeys[nr].menu_x;
  else
    *x = gdk_screen_width () - req.width - priv->softkeys[nr].menu_x;
  
  *y = gdk_screen_height() - req.height - priv->softkeys[nr].menu_y;
  *push_in = FALSE;
}

/** 
 * gpe_skwindow_softkey_popup_menu: 
 * @window: Softkey window
 * @key_number: Softkey index with popup to activate.
 *
 * Open the popup menu of a given softkey.
 */
void 
gpe_skwindow_softkey_popup_menu (GpeSKWindow *window, gint key_number)
{
  GpeSKWindowPrivate *priv;
  GtkWidget *menu;
  priv = GET_PRIVATE (window);
  
  g_return_if_fail (key_number >= 0 && key_number < window->num_softkeys);
	
  menu = priv->softkeys[key_number].menu;
    
  if ((priv->have_softkeys) && (menu))
    {
      if (!GTK_WIDGET_REALIZED (menu))
        gtk_widget_realize (menu);
      if (!GTK_WIDGET_MAPPED (menu))
        gtk_widget_show (menu);
      gtk_menu_popup (GTK_MENU (menu), NULL, NULL, 
                      softkey_menu_position, (gpointer)key_number, 0, 
                      gtk_get_current_event_time ());
      gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), TRUE);
    }
}

/** 
 * gpe_skwindow_softkey_activated: 
 * @window: Softkey window
 * @key_number: Softkey index to activate.
 *
 * Emits the #softkey_activated signal with a given softkey. Invalid softkey
 * values will be ignored.
 */
void 
gpe_skwindow_softkey_activated (GpeSKWindow *window, gint key_number)
{
  g_return_if_fail (GPE_IS_SKWINDOW (window));
 
  g_signal_emit (window, skwindow_signals[ACTIVATE_KEY], 0, key_number);
  GpeSKWindowPrivate *priv;
  priv = GET_PRIVATE (window);
  if (priv->widget_disposed)
    return;
  gpe_skwindow_softkey_popup_menu (window, key_number);
}

static gboolean
key_press_event_handler (GpeSKWindow *self, GdkEventKey *k, gpointer userdata)
{
  	
  gint i;
  GpeSKWindowPrivate *priv;
    
  priv = GET_PRIVATE (self);
  
  /* softkeys activated with all modifiers */ 
  for (i = 0; i < self->num_softkeys; i++)
    {      
      if (k->keyval == priv->softkeys[i].key)
        {
          gpe_skwindow_softkey_activated (self, i);
          return TRUE;
        }
    }
    
  return FALSE;
}

static gboolean
window_activated_handler (GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
  GpeSKWindow *window = GPE_SKWINDOW (widget);	
  GpeSKWindowPrivate *priv;

  priv = GET_PRIVATE (window);
  if (priv->have_softkeys)
    {
       gint i;
        
       for (i = 0; i < window->num_softkeys; i++)
        {
          if (priv->softkeys[i].label)
            skwindow_dbus_set_softkey_label (i, priv->softkeys[i].label);
        }
    }
  return FALSE;
}
	
static void
gpe_skwindow_init (GpeSKWindow *window)
{
  GpeSKWindowPrivate *priv;
  gint i;

  /* init private data */
  priv = GET_PRIVATE (window);
  priv->widget_disposed = FALSE;

  /* connect internal signals */
  g_signal_connect (window, "key_press_event",
                    G_CALLBACK (key_press_event_handler),
                    NULL);

  /* open connection, init softkeys */
  skwindow_dbus_init (window);
  window->num_softkeys = skwindow_dbus_get_softkey_amount ();
  priv->have_softkeys = window->num_softkeys ? TRUE : FALSE;
  
  if (priv->have_softkeys)
  {      
    priv->softkeys = g_malloc0 (window->num_softkeys * sizeof (t_softkey));
          
    for (i = 0; i < window->num_softkeys; i++)
      { 
        gchar *keyname;
        gint menu_x, menu_y;
        keyname = skwindow_dbus_get_softkey_key (i);
        priv->softkeys[i].key = gdk_keyval_from_name (keyname);
        skwindow_dbus_get_softkey_menupos (i, &menu_x, &menu_y);
        priv->softkeys[i].menu_x = menu_x;
        priv->softkeys[i].menu_y = menu_y;
      }	 

    g_signal_connect (G_OBJECT (window), "focus-in-event", 
                    G_CALLBACK (window_activated_handler), NULL);
    gtk_widget_add_events (GTK_WIDGET (window), GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
  }
    
  /* setup widget defaults */    
  gtk_window_set_type_hint (GTK_WINDOW (window), GDK_WINDOW_TYPE_HINT_NORMAL);
}


static void 
gpe_skwindow_set_property (GObject      *object,
                           guint         prop_id,
                           const GValue *value,
                           GParamSpec   *pspec)
{
  GpeSKWindow *window;
  
  window = GPE_SKWINDOW (object);
  
  switch (prop_id)
    {
      default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void 
gpe_skwindow_get_property (GObject     *object,
                           guint        prop_id,
                           GValue      *value,
                           GParamSpec  *pspec)
{
  GpeSKWindow *window;
  
  window = GPE_SKWINDOW (object);
  
  switch (prop_id)
    {
    case PROP_NUM_SOFTKEYS:
      g_value_set_int (value, window->num_softkeys);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
gpe_skwindow_map (GtkWidget *window)
{
  GpeSKWindowPrivate *priv;
    
  /* init private data */
  priv = GET_PRIVATE (window);
    
  GTK_WIDGET_CLASS (parent_class)->map (GTK_WIDGET(window));
}

/** 
 * gpe_skwindow_get_softkey_count: 
 * @window: Softkey window
 *
 * Retrieve the amount of available softkeys from a #GpeSKWindow widget.
 * The actual amount is defined by the communication peer (which is usually
 * gpe-applauncher.
 * 
 * Returns: Number of available softkeys.
 */
guint 
gpe_skwindow_get_softkey_count (GpeSKWindow *window)
{
  g_return_val_if_fail (GPE_IS_SKWINDOW (window), 0);

  return window->num_softkeys;
}


/** 
 * gpe_skwindow_get_softkey_label: 
 * @window: Softkey window
 * @key_number: Index of softkey
 *
 * Get the current label of a softkey from a #GpeSKWindow widget.
 * Invalid indices are ignored and return NULL. The returned value
 * may not be altered or freed.
 * 
 * Returns: Pointer to softkey label text if label is set, NULL otherwise.
 */
G_CONST_RETURN gchar*
gpe_skwindow_get_softkey_label (GpeSKWindow *window, gint key_number)
{
  GpeSKWindowPrivate *priv;

  g_return_val_if_fail (GPE_IS_SKWINDOW (window), NULL);

  priv = GET_PRIVATE (window);

  if (key_number < window->num_softkeys)
    return priv->softkeys[key_number].label;
  else
    return NULL;
}

/** 
 * gpe_skwindow_set_softkey_label: 
 * @window: Softkey window
 * @key_number: Index of softkey
 * @label: New label text.
 *
 * Set the current label of a softkey from a #GpeSKWindow widget.
 * Invalid indices are ignored.
 */
void 
gpe_skwindow_set_softkey_label (GpeSKWindow *window, gint key_number, const gchar *label)
{
  GpeSKWindowPrivate *priv;
  
  g_return_if_fail (GPE_IS_SKWINDOW (window));

  priv = GET_PRIVATE (window);

  if (key_number < window->num_softkeys)
    {
      if (priv->softkeys[key_number].label)
        g_free (priv->softkeys[key_number].label);
      priv->softkeys[key_number].label = g_strdup (label);
    }  
  window_activated_handler (GTK_WIDGET (window), NULL, NULL);
}


static void
menu_item_allocate (GtkWidget *widget,
	                  GtkAllocation *allocation,
	                  gpointer user_data)
{
  GtkWidget *menu_item = GTK_WIDGET(user_data);
  GtkWidget *parent = gtk_widget_get_parent (menu_item);
  GtkWidget *parentlevel = NULL;
  gint parent_x ;
  gint parent_y ;
  GtkWidget* itemtoplevel = NULL;
  gint standard_x = 0;
  gint standard_y = 0;
  GtkAllocation allo;	
	
  parentlevel = gtk_widget_get_toplevel(parent);
  gdk_window_get_root_origin (parentlevel->window,&parent_x,&parent_y);

  standard_x = parent_x - allocation->width + parent->allocation.width;

  allo = menu_item->allocation;
  standard_y =  parent_y + allo.y + parent->style->ythickness;

  if (allocation->x == standard_x && allocation->y == standard_y)
	return;
	
  itemtoplevel = gtk_widget_get_toplevel(widget);
  gtk_window_move (GTK_WINDOW(itemtoplevel), standard_x, standard_y);

  if (show_flag)
	  gtk_widget_hide(widget);
}

void 	
menu_allocate_cb (GtkWidget *menu,
                  GtkAllocation *allocation,
                  gpointer user_data)
{
  GtkWidget *menutoplevel = NULL;
	
  menutoplevel = gtk_widget_get_toplevel(menu);
  gtk_widget_set_size_request (menu, gdk_screen_width(), -1);
}                  


gboolean
menu_item_keypress_sub (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  if(event->keyval == GDK_Right)
    {
       show_flag = FALSE;
       gtk_widget_show(widget);
       gtk_menu_shell_select_first (GTK_MENU_SHELL(widget), TRUE);
       return TRUE;
    }
  else if(event->keyval == GDK_Left)
    {
      show_flag = TRUE;
      gtk_widget_hide(widget);
    }
  return FALSE;
} 


/** 
 * gpe_skwindow_set_softkey_menu: 
 * @window: Softkey window
 * @key_number: Index of softkey
 * @menu: Menu widget
 *
 * Set the current menu associated to a softkey from a #GpeSKWindow widget.
 * Invalid indices are ignored.
 */
void 
gpe_skwindow_set_softkey_menu (GpeSKWindow *window, gint key_number, GtkWidget *menu)
{
  GpeSKWindowPrivate *priv;

  g_return_if_fail (GPE_IS_SKWINDOW (window));
  
  priv = GET_PRIVATE (window);

  if (key_number < window->num_softkeys)
  {
    if (priv->softkeys[key_number].menu)
      g_object_unref(priv->softkeys[key_number].menu);
    priv->softkeys[key_number].menu = menu;
    if (menu)
  	{
  		GList* menu_items = NULL;
  		GtkMenuItem *item = NULL;
  		menu_items = gtk_container_get_children(GTK_CONTAINER(menu));

  		if(menu_fullscreen)
  		{
			  g_signal_connect(G_OBJECT(menu),
												 "size-allocate",
												 G_CALLBACK(menu_allocate_cb),
												 NULL);
  		
	  		GList *it = NULL;
				for (it = menu_items; it; it = it->next) 
				{
	      	if(GTK_MENU_ITEM(it->data)->submenu)
	  			{
						  g_signal_connect(G_OBJECT(GTK_MENU_ITEM(it->data)->submenu),
															 "size-allocate",
															 G_CALLBACK(menu_item_allocate),
															 GTK_MENU_ITEM(it->data));			
					  	g_signal_connect(G_OBJECT(GTK_MENU_ITEM(it->data)->submenu),
															 "key-press-event",
															 G_CALLBACK(menu_item_keypress_sub),
															 NULL);		
	  			}
	  		}
  		}
  		
		  g_object_ref (menu);
		  g_object_set_data (G_OBJECT (menu), "window", window);
		  g_signal_connect (G_OBJECT (menu), "hide", G_CALLBACK (menu_pop_down), GINT_TO_POINTER (key_number));
  	}
  }
}

/** 
 * gpe_skwindow_get_softkey_menu: 
 * @window: Softkey window
 * @key_number: Index of softkey
 *
 * Get the current menu widget of a softkey from a #GpeSKWindow widget.
 * Invalid indices are ignored and return NULL.
 * 
 * Returns: Window widget if widget is set, NULL otherwise.
 */
GtkWidget *
gpe_skwindow_get_softkey_menu (GpeSKWindow *window, gint key_number)
{
  GpeSKWindowPrivate *priv;

  g_return_val_if_fail (GPE_IS_SKWINDOW (window), NULL);

  priv = GET_PRIVATE (window);

  if (key_number < window->num_softkeys)
    return priv->softkeys[key_number].menu;
  else
    return NULL;
}

static void
gpe_skwindow_close (GpeSKWindow *window)
{

}

static void
gpe_skwindow_dispose (GObject *obj)
{
  GpeSKWindow *window = GPE_SKWINDOW (obj);
  GpeSKWindowPrivate *priv;
  gint i;

  priv = GET_PRIVATE (window);
  
  if (priv->widget_disposed)
      return;
  
  priv->widget_disposed = TRUE;
  
  for (i = 0; i < window->num_softkeys; i++)
      if (priv->softkeys[i].label)
        g_free (priv->softkeys[i].label);
	  
  for (i = 0; i < window->num_softkeys; i++)
      if (priv->softkeys[i].menu)
        g_object_unref (priv->softkeys[i].menu);
  g_free (priv->softkeys);
    
  G_OBJECT_CLASS (parent_class)->dispose (obj);
}

static void
gpe_skwindow_finalize (GObject *obj)
{
  G_OBJECT_CLASS (parent_class)->finalize (obj);
}

/** 
 * gpe_skwindow_new: 
 *
 * Create a new #GpeSKWindow instance. The communication to the softkey 
 * service provider is initiated automatically on object creation.
 * 
 * Returns: A new #GpeSKWindow instance.
 */
GtkWidget *
gpe_skwindow_new (void)
{
  return g_object_new (GPE_TYPE_SKWINDOW, NULL);
}

/** 
 * gpe_skwindow_set_menu_fullscreen: 
 * @full: Desired fullscreen mode.
 *
 * Change #GpeSKWindow softkey manu fullscreen mode.
 * 
 */
void
gpe_skwindow_set_menu_fullscreen(gboolean full)
{
	menu_fullscreen = full;
}

